/* Functions for drawing stuff on the screen (actually they all draw to vram)
all sprite functions are a modified version of
void CopySprite(const color_t* sprite, int x, int y, int width, int height) {
	color_t* VRAM = (color_t*)GetVRAMAddress();
	VRAM += LCD_WIDTH_PX*y + x;
	for(int j=y; j<y+height; j++) {
		memcpy(VRAM, sprite, width*sizeof(color_t));
		sprite += width;
		VRAM += LCD_WIDTH_PX;
	}
}
which is a modified version of https://prizm.cemetech.net/index.php?title=CopySprite
or the CopySprite from https://www.cemetech.net/forum/viewtopic.php?t=6114

If yo want to make games for the calculator you should get used to writing your own drawing functions,
which copy color values to the VRAM. You might also want to write your own converters using functions like
color_t RGBtoColor(unsigned char r, unsigned char g, unsigned char b){
	return (r/8*2048)+(g/4*32)+(b/8);
}
(btw. (0,0) is top-left)
*/

#include "drawing.h"

#include <fxcg/display.h>
// #include <string.h>	// memcpy
#include "memcopy.h" // memcpy replacement

// a global pointer to the VRAM-address, because we don't want to call GetVRAMAddress() every draw call
color_t* vramadress;

// initialize the global vramaddress variable
void initDrawing() {
	/* People used to have a hardcoded VRAM-Address which made the programms incompatible
	for either Casio fx-CG50 (if 0xA8000000 was used) or the fx-CG10 and fx-CG20 (if 0xAC000000 was used) */
	vramadress = (color_t*)GetVRAMAddress();
}

// Simply drawing sprites (not used in this project)
void drawSprite(const color_t* sprite, int x, int y, int width, int height) {
	color_t* VRAM = (color_t*)vramadress;
	VRAM += LCD_WIDTH_PX*y + x;
	for(int j=y; j<y+height; j++) {
		memcopy(VRAM, sprite, width*sizeof(color_t));
		VRAM += LCD_WIDTH_PX;
		sprite += width;
	}
}

/* draw a sprite with a mask color
sprite: pointer to an array of colors (the pixels of the image)
x: x-position
y: y-position
width: the width of the source image
height: the height of the source image
maskColor: the color to be transparent
*/
void drawSpriteMasked(const color_t* sprite, int x, int y, int width, int height, color_t maskColor) {
	color_t* VRAM = vramadress;
	VRAM += LCD_WIDTH_PX*y + x;
	for(int j=y; j<y+height; j++) {
		for(int i=x; i<x+width; i++) {
			if(*sprite != maskColor)	// check whether the colour should actually be drawn
		 		*VRAM = *sprite;
			sprite++;
			VRAM++;
	  	}
		VRAM += LCD_WIDTH_PX-width;
   	}
}

/* draw a sprite which is cut off by the edge of the screen
(because we don't want our sprites to wrap around to the other side of the screen) */
void drawSpriteSafe(const color_t* sprite, int x, int y, int width, int height) {
	color_t* VRAM = vramadress;

	// left cutoff
	unsigned int x_offset = 0;
	if(x < 0) x_offset = -x;

	// right cutoff
	int cutwidth = width-x_offset;
	if(x+width > LCD_WIDTH_PX)
		cutwidth -= (x+width)-LCD_WIDTH_PX;
	if(cutwidth < 1) return;	// the sprite is out of view

	// top cutoff
	unsigned int y_offset = 0;
	if(y < 0) y_offset = -y;

	// bottom cutoff
	if(y+height > LCD_HEIGHT_PX)
		height -= (y+height)-LCD_HEIGHT_PX;
	
	VRAM += LCD_WIDTH_PX*(y+y_offset) + x+x_offset;
	sprite += y_offset*width+x_offset;
	for(int j=y+y_offset; j<y+height; j++) {
		memcopy(VRAM, sprite, cutwidth*sizeof(color_t));
		sprite += width;
		VRAM += LCD_WIDTH_PX;
	}
}
